//////////////////////////////////////////////////////////////////////////////
// This implementation of LDKL was written by Suraj Jain. Please contact    //
// Suraj Jain <sujain@microsoft.com> and Manik Varma <manik@microsoft.com>  //
// for questions or feedback.                                               //
////////////////////////////////////////////////////////////////////////////// 
#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include "string.h"
#include <math.h>
#include <vector>
#include"../include/Utils.h"

using namespace std;

//Log2
unsigned int log2( unsigned int x )
{
  unsigned int ans = 0 ;
  while( x>>=1 ) ans++;
  return ans ;
}

// Compute L2 norm of Vector

MyFloat dnrm2_(const MyFloat *v, int n) {
  MyFloat ntemp = 0.0;
  for(int i=0; i<n; i++) 
    ntemp += pow(v[i], 2);
  return ntemp;
} 

// Average of numbers

MyFloat avg(MyFloat *Data, int N){
	MyFloat sum = 0;
	for(int i = 0 ; i < N ; i++)
		sum += Data[i];
	return(sum/N);
}
double avg(double *Data, int N){
	double sum = 0;
	for(int i = 0 ; i < N ; i++)
		sum += Data[i];
	return(sum/N);
}

// Load Data

void exit_with_readError(int num, char* filename){
	cout << "Error in Reading Line " << num << " in file " << filename << endl;
	exit(1);
}
#if SPARSE
	void ReadData(char* Filename,DATA &X, LABEL &Y, int &Dim ,int &N){
	int count, max_index, inst_max_index;
	char line[100000];
	char *label,*idx,*val, *endptr;
	int tempidx;
	float tempval;
	max_index = -1;
	vector<MyFloat> labels;
	vector<vector<Node>> Data;
	count = 0;
	FILE* datafile = fopen(Filename,"r");
	if(datafile == NULL)
	{
		cout << "Can't open input file "<< Filename << endl;
		exit(1);
	}
	while(fgets(line,1000000,datafile) != NULL){
			vector<Node> TempVec;
			inst_max_index = -1;
			//Get Label
			label = strtok(line," \t");
			if(label == NULL)
				exit_with_readError(count+1,Filename);
			labels.push_back((MyFloat)strtod(label,&endptr));
			if(endptr == label || *endptr !='\0')
				exit_with_readError(count+1,Filename);
			//Get Instance
			while(1)
			{
				idx = strtok(NULL,":");
				val = strtok(NULL," \t");

				if(val == NULL)
					break;
	
				tempidx = (int) strtol(idx,&endptr,10);
				if(endptr == idx || *endptr != '\0' || tempidx <= inst_max_index)
					exit_with_readError(count+1,Filename);
				else
					inst_max_index = tempidx;

			    tempval = (MyFloat)strtod(val,&endptr);
				if(endptr == val || (*endptr != '\0' && !isspace(*endptr)))
					exit_with_readError(count+1,Filename);
				Node n;
				n.idx = tempidx;
				n.val = tempval;
				TempVec.push_back(n);
			}
			if(inst_max_index > max_index)
				max_index = inst_max_index;
			count++;
			Data.push_back(TempVec);
	}
	N = count;
	Dim = max_index + 1;
	Y = new MyFloat[labels.size()];
	X = new INSTANCE[count];
	for(int i = 0 ; i < count ; i++)
		Y[i] = labels[i];
	for(int i = 0 ; i < count ; i++){
		X[i].count = Data[i].size() + 1;
		X[i].idx = new int[X[i].count];
		X[i].val = new MyFloat[X[i].count];
		for(unsigned int j = 0 ; j < Data[i].size() ; j++){
			X[i].idx[j] = Data[i][j].idx - 1; // Index is zero based
			X[i].val[j] = Data[i][j].val;
		}
		//Add Bias and sentinal
		X[i].idx[X[i].count-1] = Dim - 1;
		X[i].val[X[i].count-1] = 1;
	}
	fclose(datafile);
}
#else
	void ReadData(char* FileName, DATA &X, MyFloat* &y,int &D, int &N){
		FILE* Fp;
		char* tok;
		char Line[100000];
		int Ncount = 0;
		int Dcount = 0;
		Fp = fopen(FileName,"r");
		if(Fp == NULL){
			cout << "Error in Opening file: " << FileName << endl;
			exit(0);
		}
		if(fgets(Line,100000,Fp) != NULL){
			tok = strtok(Line," ");
			N = atoi(tok);
			tok = strtok(NULL," ");
			D = atoi(tok)+1;
		}	
		X = new INSTANCE[N];
		y = new MyFloat[N];
		while(fgets(Line,100000,Fp) != NULL){
			X [Ncount] = new MyFloat[D];
			tok = strtok(Line," ");
			y[Ncount] = (MyFloat)atof(tok);
			tok = strtok(NULL," ");
			while(tok != NULL){
				X[Ncount][Dcount] = (MyFloat)atof(tok);
				Dcount++;
				tok = strtok(NULL," ");
			}
			X[Ncount][Dcount] = 1.0;
			Dcount++;
			if(Dcount != D){
				cout << "Dimension Mismatch while reading data from file: "<< FileName << " at line number: " << Ncount+2<<endl;
				exit(1);
			}
			Ncount++;
			Dcount = 0;
		}
		if(Ncount != N){
			cout<< "Mismatch in total number of datapoints in file: "<< FileName<<endl;
			exit(2);
		}
		fclose(Fp);
	}
#endif